// console.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>
#include <stdio.h>

extern "C" {
  #include "usbi2cio.h"
}

void IoExample(HANDLE hDevInstance);
void I2cExample(HANDLE hDevInstance);

int main(int argc, char* argv[]) {

	HANDLE hDevInstance = INVALID_HANDLE_VALUE; // device instance handle

	// attempt to open device UsbI2cIo0 (instance 0 of the device with symbolic name UsbI2cIo)
	hDevInstance = DAPI_OpenDeviceInstance("UsbI2cIo", 0);

	// check that we actually got a good handle
	if( hDevInstance != INVALID_HANDLE_VALUE) {
		// success - device instance handle opened
		printf("Opened device UsbI2cIo0\n");

		// Call the IoExample function (see IoExample function definition below)
		printf("\nCalling IoExample function\n\n");
		IoExample(hDevInstance);

		// Call the I2cExample function (see IoExample function definition below)
		printf("\nCalling I2cExample function\n\n");
		I2cExample(hDevInstance);

		// close the handle to device instance
		printf("\nClosing handle to board\n\n");
		DAPI_CloseDeviceInstance(hDevInstance);
	}
	else {
		// we were unable to open a handle to the USB-I2C/IO board
		printf("Failed to open a handle to device UsbI2cIo0\n");
	}

	return 0;
}


void IoExample(HANDLE hDevInstance) {
		
	// This function demonstrates I/O functionality by configuring, writing and reading the I/O pins.

	// Please refer to the API User's Guide for detailed function calling

	// Pins are bit configurable, any pin can be an input or an output
	// Pins default to inputs on power-up, power-up states can be set with external pull-up or pull-down resistors
		
	ULONG ulPins;        // variable used to hold value read from I/O pins
		
	// configure I/O Port C as Inputs, and I/O Ports B and A as Outputs
	// BOOL _stdcall DAPI_ConfigIoPorts(HANDLE hDevInstance, ULONG ulIoPortConfig);
	DAPI_ConfigIoPorts(hDevInstance, 0x000F0000); // bits that are: 0 = output, 1 = input
	printf("Configured I/O Port C as Inputs, and I/O Ports B and A as Outputs\n");

	// clear all I/O Port B and A output bits except A.0 which will be set
	// I/O Port C bits are unaffected as they were configured as inputs
	// BOOL _stdcall DAPI_WriteIoPorts(HANDLE hDevInstance, ULONG ulIoPortData, ULONG ulIoPortMask);
	DAPI_WriteIoPorts(hDevInstance, 0x00000001, 0xFFFFFFFF);
	printf("Cleared I/O Ports B and A to 0, with the exception of A.0 which was set to 1\n");

	// read the pins
	// NOTE: Any pins configured as inputs and left unconnected will float to unknown values
	// BOOL _stdcall DAPI_ReadIoPorts(HANDLE hDevInstance, LPLONG lpulIoPortData);
	DAPI_ReadIoPorts(hDevInstance, (LONG *)&ulPins);
	printf("Read value 0x%08X from the I/O pins\n", ulPins);

	// now set bit B.7, and clear bit A.0, leave other bits unmodified
	// BOOL _stdcall DAPI_WriteIoPorts(HANDLE hDevInstance, ULONG ulIoPortData, ULONG ulIoPortMask);
	DAPI_WriteIoPorts(hDevInstance, 0x00008000, 0x00008001);
	printf("Set bit B.7 and cleared bit A.0, left other bits unmodified\n");

	// read the pins
	// NOTE: Any pins configured as inputs and left unconnected will float to unknown values
	// BOOL _stdcall DAPI_ReadIoPorts(HANDLE hDevInstance, LPLONG lpulIoPortData);
	DAPI_ReadIoPorts(hDevInstance, (LONG *)&ulPins);
	printf("Read value 0x%08X from the I/O pins\n", ulPins);

	// configure all I/O pins back to inputs
	// BOOL _stdcall DAPI_ConfigIoPorts(HANDLE hDevInstance, ULONG ulIoPortConfig);
	DAPI_ConfigIoPorts(hDevInstance, 0x000FFFFF); // bits that are: 0 = output, 1 = input
	printf("Configured all I/O pins as Inputs\n");

}

void I2cExample(HANDLE hDevInstance) {

	// This function demontrates I2C functionality by reading the board serial number from the onboard eeprom

	// This example will read an 8 byte serial number (ASCII) from the onboard I2C eeprom

	I2C_TRANS I2cTrans;		// An instance of an I2C Transaction Structure, used for I2C operations
	LONG lRetVal;					// variable used to hold value returned from function calls
	int i;								// for loop indexing

	// Initialize the I2C structure for the type of I2C transaction we want to perform
	// Structure definition shown here for easy reference:

	/*
		typedef struct _I2C_TRANS {
			BYTE byTransType;
			BYTE bySlvDevAddr;
			WORD wMemoryAddr;
			WORD wCount;
			BYTE Data[256];
		} I2C_TRANS, *PI2C_TRANS;
	*/


	// The onboard eeprom uses special I2C protocol for memory devices with 2 byte addresses
	// this is an internal address within the devices memory array, not the I2C bus address.
	// So, to have the USB-I2C/IO automatically perform the special I2C protocol, we specify
	// a transaction type of I2C_TRANS_16ADR when communicating with the onboard eeprom.
	// Other type of I2C devices may use one of the other transaction types.
	// Transaction types shown here for easy reference:

	/*
		typedef enum {
			// supported transaction types
			I2C_TRANS_NOADR,          // read or write with no address cycle
			I2C_TRANS_8ADR,           // read or write with 8 bit address cycle
			I2C_TRANS_16ADR,          // read or write with 16 bit address cycle
			I2C_TRANS_NOADR_NS        // read or write with no address cycle, stop signaling suppressed
		} I2C_TRANS_TYPE;
	*/

	// Specify the I2C Transaction type for this transaction.
	I2cTrans.byTransType = I2C_TRANS_16ADR;

	// Specify the I2C bus address of the onboard eeprom
	I2cTrans.bySlvDevAddr = 0xA2;

	// Specify the address within the onboard eeprom memory array at which we will begin our access
	I2cTrans.wMemoryAddr = 0x3F00;

	// Specify the number of bytes we will read from the onboard I2C eeprom
	I2cTrans.wCount = 8;

	// if we were doing an I2C write, we would now initialize I2cTrans.Data, but we are doing a read

	// Perform the I2C read transaction
	// LONG _stdcall DAPI_ReadI2c(HANDLE hDevInstance, PI2C_TRANS TransI2C);
	lRetVal = DAPI_ReadI2c(hDevInstance, &I2cTrans);

	// check the return value to determine is transaction was successful
	if (lRetVal == I2cTrans.wCount) {
		// great, we requested 8, and got 8

		// display the data we read from the eeprom (which should be the board serial number)
		printf("Read 8 bytes from memory address 0x3F00 of 24LC128 eeprom at I2C address 0xA2\n");
		printf("SN:");
		for(i=0; i<8; i++) {
			// display each byte of data, remember... these are ASCII bytes (characters)
			printf("%c",I2cTrans.Data[i]);
		}
		printf("\n");
	}
	else {
		// error, we didn't get what we asked for
		printf("error - DAPI_ReadI2c returned %d, expected %d\n", lRetVal, I2cTrans.wCount);
	}

	// The I2C functionality supports a wide variety of I2C peripherals
	
	//  I2C Writes were not demonstrated, but are very similar to reads.
	//  The main difference is that the I2cTrans.Data array is initialized BEFORE making the DAPI_WriteI2c function call

	// We highly recommend that you DO NOT WRITE to the onboard eeprom unless you know exactly what you are doing.
	// If you overwrite certain areas of the onboard eeprom, the board will not work correctly.
	// If this happens... it is recoverable via a utility... but it's best not to do it.

}
